# Copyright 2017 NTT
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json
from oslo_utils import uuidutils as uuids
from tempest.lib.common import rest_client


class ResourceReservationV1Client(rest_client.RestClient):
    """Client class for accessing the resource reservation API."""
    BLAZARCLIENT_VERSION = '1'

    lease = '/leases'
    lease_path = '/leases/%s'
    host = '/os-hosts'
    host_path = '/os-hosts/%s'

    def _response_helper(self, resp, body=None):
        if body:
            body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def _get_headers(self):
        headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            'X-Openstack-Request-Id': 'req-' + str(uuids.generate_uuid())
        }
        return headers

    def list_lease(self):
        resp, body = self.get(self.lease, headers=self._get_headers())
        return self._response_helper(resp, body)

    def get_lease(self, lease):
        resp, body = self.get(self.lease_path % lease)
        return self._response_helper(resp, body)

    def create_lease(self, body):
        body = json.dump_as_bytes(body)
        resp, body = self.post(
            self.lease, body=body, headers=self._get_headers())
        return self._response_helper(resp, body)

    def update_lease(self, lease, body):
        body = json.dump_as_bytes(body)
        resp, body = self.put(
            self.lease_path % lease, body=body, headers=self._get_headers())
        return self._response_helper(resp, body)

    def delete_lease(self, lease):
        resp, body = self.delete(
            self.lease_path % lease, headers=self._get_headers())
        return self._response_helper(resp, body)

    def list_host(self):
        resp, body = self.get(self.host, headers=self._get_headers())
        return self._response_helper(resp, body)

    def get_host(self, host):
        resp, body = self.get(
            self.host_path % host, headers=self._get_headers())
        return self._response_helper(resp, body)

    def create_host(self, body):
        body = json.dump_as_bytes(body)
        resp, body = self.post(
            self.host, body=body, headers=self._get_headers())
        return self._response_helper(resp, body)

    def update_host(self, host, body):
        body = json.dump_as_bytes(body)
        resp, body = self.put(
            self.host_path % host, body=body, headers=self._get_headers())
        return self._response_helper(resp, body)

    def delete_host(self, host):
        resp, body = self.delete(
            self.host_path % host, headers=self._get_headers())
        return self._response_helper(resp, body)



# Copyright 2017 University of Chicago. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from multiprocessing.pool import ThreadPool

from blazar_tempest_plugin.tests.scenario import (
    resource_reservation_scenario as rrs)


class TestReservationConcurrencyScenario(rrs.ResourceReservationScenarioTest):
    """A Scenario test class checking Blazar handles concurrent requests."""

    MAX_CONCURRENCY = 10

    def setUp(self):
        super(TestReservationConcurrencyScenario, self).setUp()

    def tearDown(self):
        super(TestReservationConcurrencyScenario, self).tearDown()

    def test_concurrent_list_lease(self):
        # run lease-list requests in parallel to check service concurrency
        results = []
        pool = ThreadPool(self.MAX_CONCURRENCY)
        for i in range(0, self.MAX_CONCURRENCY):
            results.append(
                pool.apply_async(self.reservation_client.list_lease, ()))

        pool.close()
        pool.join()
        results = [r.get() for r in results]
        for r in results:
            self.assertEqual('200', r.response['status'])



# Copyright 2014 Intel Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime

from oslo_log import log as logging
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators

from blazar_tempest_plugin.tests.scenario import (
    resource_reservation_scenario as rrs)

CONF = config.CONF

LOG = logging.getLogger(__name__)

# same as the one at blazar/manager/service
LEASE_DATE_FORMAT = "%Y-%m-%d %H:%M"


class TestInstanceReservationScenario(rrs.ResourceReservationScenarioTest):
    """A scenario test class that checks the instance reservation feature."""

    def setUp(self):
        super(TestInstanceReservationScenario, self).setUp()
        # Setup image and flavor the test instance
        # Support both configured and injected values
        if not hasattr(self, 'image_ref'):
            self.image_ref = CONF.compute.image_ref
        if not hasattr(self, 'flavor_ref'):
            self.flavor_ref = CONF.compute.flavor_ref
        if not self.is_flavor_enough(self.flavor_ref, self.image_ref):
            raise self.skipException(
                '{image} does not fit in {flavor}'.format(
                    image=self.image_ref, flavor=self.flavor_ref
                )
            )
        self.host = self._add_host_once()

    def tearDown(self):
        super(TestInstanceReservationScenario, self).tearDown()

    def get_lease_body(self, lease_name):
        current_time = datetime.datetime.utcnow()
        end_time = current_time + datetime.timedelta(hours=1)
        body = {
            "start_date": "now",
            "end_date": end_time.strftime(LEASE_DATE_FORMAT),
            "name": lease_name,
            "events": [],
            }
        body["reservations"] = [
            {
                "resource_type": 'virtual:instance',
                'vcpus': 1,
                'memory_mb': 1024,
                'disk_gb': 10,
                'amount': 1,
                'affinity': False,
                'resource_properties': '',
                }
            ]
        return body

    def get_lease_expiration_body(self, lease_name):
        current_time = datetime.datetime.utcnow()
        end_time = current_time + datetime.timedelta(seconds=90)
        body = {
            "start_date": "now",
            "end_date": end_time.strftime(LEASE_DATE_FORMAT),
            "name": lease_name,
            "events": [],
            }
        body["reservations"] = [
            {
                "resource_type": 'virtual:instance',
                'vcpus': 1,
                'memory_mb': 1024,
                'disk_gb': 10,
                'amount': 1,
                'affinity': False,
                'resource_properties': '',
                }
            ]
        return body

    @decorators.attr(type='smoke')
    def test_instance_reservation(self):
        body = self.get_lease_body('instance-scenario')
        lease = self.reservation_client.create_lease(body)['lease']
        reservation = next(iter(lease['reservations']))

        self.wait_for_lease_status(lease['id'], 'ACTIVE')

        # create an instance within the reservation
        create_kwargs = {
            'image_id': CONF.compute.image_ref,
            'flavor': reservation['id'],
            'scheduler_hints': {
                'group': reservation['server_group_id']
                },
            }
        server1 = self.create_server(clients=self.os_admin,
                                     **create_kwargs)

        # create another instance within the reservation, which is expected to
        # fail because we are over the reserved capacity
        create_kwargs = {
            'image_id': CONF.compute.image_ref,
            'flavor': reservation['id'],
            'scheduler_hints': {
                'group': reservation['server_group_id']
                },
            }
        server2 = self.create_server(clients=self.os_admin,
                                     wait_until=None,
                                     **create_kwargs)
        waiters.wait_for_server_status(self.os_admin.servers_client,
                                       server2['id'], 'ERROR',
                                       raise_on_error=False)

        # create an instance without specifying a reservation, which is
        # expected to fail
        create_kwargs = {
            'image_id': CONF.compute.image_ref,
            'flavor': CONF.compute.flavor_ref,
            }
        server3 = self.create_server(clients=self.os_admin,
                                     wait_until=None,
                                     **create_kwargs)
        waiters.wait_for_server_status(self.os_admin.servers_client,
                                       server3['id'], 'ERROR',
                                       raise_on_error=False)

        # delete the lease, which should trigger termination of the instance
        self.reservation_client.delete_lease(lease['id'])
        waiters.wait_for_server_termination(self.os_admin.servers_client,
                                            server1['id'])

    @decorators.attr(type='smoke')
    def test_lease_expiration(self):
        body = self.get_lease_expiration_body('instance-scenario-expiration')
        lease = self.reservation_client.create_lease(body)['lease']
        reservation = next(iter(lease['reservations']))

        self.wait_for_lease_status(lease['id'], 'ACTIVE')

        create_kwargs = {
            'image_id': CONF.compute.image_ref,
            'flavor': reservation['id'],
            'scheduler_hints': {
                'group': reservation['server_group_id']
                },
            }
        server1 = self.create_server(clients=self.os_admin,
                                     **create_kwargs)

        # wait for lease end
        self.wait_for_lease_end(lease['id'])

        waiters.wait_for_server_termination(self.os_admin.servers_client,
                                            server1['id'])

        # check the lease status and reservation status
        lease = self.reservation_client.get_lease(lease['id'])['lease']
        self.assertTrue(lease['status'] == 'TERMINATED')
        self.assertTrue('deleted' in
                        next(iter(lease['reservations']))['status'])

    @decorators.attr(type='smoke')
    def test_update_instance_reservation(self):
        body = self.get_lease_body('instance-scenario-update')
        lease = self.reservation_client.create_lease(body)['lease']
        reservation = next(iter(lease['reservations']))

        self.wait_for_lease_status(lease['id'], 'ACTIVE')

        create_kwargs = {
            'image_id': CONF.compute.image_ref,
            'flavor': reservation['id'],
            'scheduler_hints': {
                'group': reservation['server_group_id']
                },
            }
        server = self.create_server(clients=self.os_admin,
                                    **create_kwargs)

        # Updating the lease end_date to 1 minute from now to avoid a failure
        # of the lease update request
        now = datetime.datetime.utcnow()
        end_time = now + datetime.timedelta(minutes=1)
        body = {
            'end_date': end_time.strftime(LEASE_DATE_FORMAT)
            }
        self.reservation_client.update_lease(lease['id'], body)

        waiters.wait_for_server_termination(self.os_admin.servers_client,
                                            server['id'])

        # There is a lag between the server termination and the lease status
        # transition. Let's wait a bit here.
        self.wait_for_lease_end(lease['id'])

        # check the lease status and reservation status
        lease = self.reservation_client.get_lease(lease['id'])['lease']
        self.assertTrue(lease['status'] == 'TERMINATED')
        self.assertTrue('deleted' in
                        next(iter(lease['reservations']))['status'])



# Copyright 2014 Intel Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from oslo_log import log
from tempest import clients as tempestclients
from tempest import config
from tempest import exceptions
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions as lib_exceptions

from blazar_tempest_plugin.services.reservation import (
    reservation_client as clients)
from blazar_tempest_plugin.tests.scenario import manager_freeze as manager

CONF = config.CONF

LOG = log.getLogger(__name__)


class ResourceReservationScenarioTest(manager.ScenarioTest):
    """Base class for resource reservation scenario tests."""

    credentials = ['primary', 'admin']

    @classmethod
    def setup_clients(cls):
        super(ResourceReservationScenarioTest, cls).setup_clients()
        if not CONF.service_available.blazar:
            raise cls.skipException("Resource reservation support is"
                                    "required")

        cred_provider = cls._get_credentials_provider()
        creds = cred_provider.get_credentials('admin')
        auth_prov = tempestclients.get_auth_provider(creds._credentials)
        cls.os_admin.resource_reservation_client = (
            clients.ResourceReservationV1Client(auth_prov,
                                                'reservation',
                                                CONF.identity.region))
        cls.reservation_client = (
            cls.os_admin.resource_reservation_client)

    def get_lease_by_name(self, lease_name):
        # the same as the blazarclient does it: ask for the entire list
        lease_list = self.reservation_client.list_lease()
        named_lease = []

        # and then search by lease_name
        named_lease = [lease for lease in lease_list
                       if lease['name'] == lease_name]

        if named_lease:
            return self.reservation_client.get_lease(
                named_lease[0]['id'])
        else:
            message = "Unable to find lease with name '%s'" % lease_name
            raise exceptions.NotFound(message)

    def delete_lease(self, lease_id):
        return self.reservation_client.delete_lease(lease_id)

    def wait_for_lease_end(self, lease_id):

        def check_lease_end():
            try:
                lease = self.reservation_client.get_lease(lease_id)['lease']
                if lease:
                    events = lease['events']
                    return len([evt for evt in events
                                if evt['event_type'] == 'end_lease' and
                                evt['status'] == 'DONE']) > 0
                else:
                    LOG.info("Lease with id %s is empty", lease_id)
            except Exception as e:
                LOG.info("Unable to find lease with id %(lease_id)s. "
                         "Exception: %(message)s",
                         {'lease_id': lease_id, 'message': str(e)})
            return True

        if not test_utils.call_until_true(
            check_lease_end,
            CONF.resource_reservation.lease_end_timeout,
                CONF.resource_reservation.lease_interval):
            message = "Timed out waiting for lease to change status to DONE"
            raise lib_exceptions.TimeoutException(message)

    def wait_for_lease_status(self, lease_id, status):

        def check_lease_status():
            try:
                lease = self.reservation_client.get_lease(lease_id)['lease']
                if lease and lease['status'] == status:
                    return True
                else:
                    LOG.info("Lease with id %s is not %s, but %s",
                             lease_id, status, lease['status'])
            except Exception as e:
                LOG.info("Unable to find lease with id %(lease_id)s. "
                         "Exception: %(message)s",
                         {'lease_id': lease_id, 'message': str(e)})
            return False

        if not test_utils.call_until_true(
            check_lease_status,
            CONF.resource_reservation.lease_end_timeout,
                CONF.resource_reservation.lease_interval):
            message = ("Timed out waiting for lease to change status "
                       "to %s" % status)
            raise lib_exceptions.TimeoutException(message)

    def is_flavor_enough(self, flavor_id, image_id):
        image = self.compute_images_client.show_image(image_id)['image']
        flavor = self.flavors_client.show_flavor(flavor_id)['flavor']
        return image['minDisk'] <= flavor['disk']

    def fetch_one_compute_host(self):
        """Returns the first host listed in nova-compute services."""
        compute = next(iter(self.os_admin.services_client.
                            list_services(binary='nova-compute')['services']))
        return compute

    def _add_host_once(self):
        host = self.fetch_one_compute_host()
        hosts = self.reservation_client.list_host()['hosts']
        try:
            # TODO(masahito): Fix the check not to depend on '0' fixed check.
            # When the scenario test covers the monitor resource feature, there
            # are multiple compute hosts.
            [h for h in hosts if h['hypervisor_hostname'] == host['host']][0]
        except IndexError:
            self.reservation_client.create_host({'name': host['host']})
        return host



# Copyright 2017 NTT Corporation. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import time

from oslo_log import log as logging
from six.moves import range
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions

from blazar_tempest_plugin.tests.scenario import (
    resource_reservation_scenario as rrs)

CONF = config.CONF

LOG = logging.getLogger(__name__)


class TestHostReservationScenario(rrs.ResourceReservationScenarioTest):
    """A Scenario test class that checks host reservation."""

    MAX_RETRY = 20
    WAIT_TIME = 2

    def setUp(self):
        super(TestHostReservationScenario, self).setUp()
        self.aggr_client = self.os_admin.aggregates_client

    def tearDown(self):
        super(TestHostReservationScenario, self).tearDown()

    def get_lease_body(self, lease_name, host_name):
        current_time = datetime.datetime.utcnow()
        end_time = current_time + datetime.timedelta(hours=1)
        body = {
            "start_date": "now",
            "end_date": end_time.strftime('%Y-%m-%d %H:%M'),
            "name": lease_name,
            "events": [],
            }
        body["reservations"] = [
            {
                "hypervisor_properties": ('["==", "$hypervisor_hostname", "'
                                          '%s"]' % host_name),
                "max": 1,
                "min": 1,
                "resource_type": 'physical:host',
                "resource_properties": ''
                }
            ]

        return body

    def get_lease_body_missing_param(self, lease_name, host_name):
        current_time = datetime.datetime.utcnow()
        end_time = current_time + datetime.timedelta(hours=1)
        body = {
            "start_date": "now",
            "end_date": end_time.strftime('%Y-%m-%d %H:%M'),
            "name": lease_name,
            "events": [],
            }
        body["reservations"] = [
            {
                "hypervisor_properties": ('["==", "$hypervisor_hostname", "'
                                          '%s"]' % host_name),
                "min": '1',
                "resource_type": 'physical:host',
                "resource_properties": ''
                }
            ]

        return body

    def get_invalid_lease_body(self, lease_name, host_name):
        current_time = datetime.datetime.utcnow()
        end_time = current_time + datetime.timedelta(hours=1)
        body = {
            "start_date": "now",
            "end_date": end_time.strftime('%Y-%m-%d %H:%M'),
            "name": lease_name,
            "events": [],
            }
        body["reservations"] = [
            {
                "hypervisor_properties": ('["==", "$hypervisor_hostname", "'
                                          '%s"]' % host_name),
                "max": 'foo',
                "min": 'bar',
                "resource_type": 'physical:host',
                "resource_properties": ''
                }
            ]

        return body

    def get_expiration_lease_body(self, lease_name, host_name):
        current_time = datetime.datetime.utcnow()
        end_time = current_time + datetime.timedelta(seconds=90)
        body = {
            'start_date': "now",
            'end_date': end_time.strftime('%Y-%m-%d %H:%M'),
            'name': lease_name,
            'events': [],
            }
        body['reservations'] = [
            {
                'hypervisor_properties': ('["==", "$hypervisor_hostname", "'
                                          '%s"]' % host_name),
                'max': 1,
                'min': 1,
                'resource_type': 'physical:host',
                'resource_properties': ''
                }
            ]

        return body

    def fetch_aggregate_by_name(self, name):
        aggregates = self.aggr_client.list_aggregates()['aggregates']
        try:
            aggr = [aggr for aggr in aggregates if aggr['name'] == name][0]
        except IndexError:
            err_msg = "aggregate with name %s doesn't exist." % name
            raise exceptions.NotFound(err_msg)
        return aggr

    def wait_until_aggregated(self, aggregate_name, host_name):
        for i in range(self.MAX_RETRY):
            try:
                aggr = self.fetch_aggregate_by_name(aggregate_name)
                self.assertTrue(host_name in aggr['hosts'])
                return
            except Exception:
                pass
            time.sleep(self.WAIT_TIME)
        err_msg = ("hostname %s doesn't exist in aggregate %s."
                   % (host_name, aggregate_name))
        raise exceptions.NotFound(err_msg)

    @decorators.attr(type='smoke')
    def test_host_reservation(self):

        # Create the host if it doesn't exists
        host = self._add_host_once()

        # check the host is in freepool
        freepool = self.fetch_aggregate_by_name('freepool')
        self.assertTrue(host['host'] in freepool['hosts'])

        # try creating a new lease with a missing parameter
        body = self.get_lease_body_missing_param('scenario-1-missing-param',
                                                 host['host'])
        self.assertRaises(exceptions.BadRequest,
                          self.reservation_client.create_lease, body)

        # try creating a new lease with an invalid request
        body = self.get_invalid_lease_body('scenario-1-invalid', host['host'])
        self.assertRaises(exceptions.BadRequest,
                          self.reservation_client.create_lease, body)

        # create new lease and start reservation immediately
        body = self.get_lease_body('scenario-1', host['host'])
        lease = self.reservation_client.create_lease(body)['lease']

        # check host added to the reservation
        reservation_id = next(iter(lease['reservations']))['id']
        self.wait_until_aggregated(reservation_id, host['host'])

        # create an instance with reservation id
        create_kwargs = {
            'scheduler_hints': {
                "reservation": reservation_id,
                },
            'image_id': CONF.compute.image_ref,
            'flavor': CONF.compute.flavor_ref,
            }
        server = self.create_server(clients=self.os_admin,
                                    **create_kwargs)
        # ensure server is located on the requested host
        self.assertEqual(host['host'], server['OS-EXT-SRV-ATTR:host'])

        # delete the lease, which should trigger termination of the instance
        self.reservation_client.delete_lease(lease['id'])
        waiters.wait_for_server_termination(self.os_admin.servers_client,
                                            server['id'])

        # create an instance without reservation id, which is expected to fail
        create_kwargs = {
            'image_id': CONF.compute.image_ref,
            'flavor': CONF.compute.flavor_ref,
            }
        server = self.create_server(clients=self.os_admin,
                                    wait_until=None,
                                    **create_kwargs)
        waiters.wait_for_server_status(self.os_admin.servers_client,
                                       server['id'], 'ERROR',
                                       raise_on_error=False)

    @decorators.attr(type='smoke')
    def test_lease_expiration(self):

        # create the host if it doesn't exist
        host = self._add_host_once()

        # create new lease and start reservation immediately
        body = self.get_expiration_lease_body('scenario-2-expiration',
                                              host['host'])
        lease = self.reservation_client.create_lease(body)['lease']
        lease_id = lease['id']

        # check host added to the reservation
        reservation_id = next(iter(lease['reservations']))['id']
        self.wait_until_aggregated(reservation_id, host['host'])

        create_kwargs = {
            'scheduler_hints': {
                'reservation': reservation_id,
                },
            'image_id': CONF.compute.image_ref,
            'flavor': CONF.compute.flavor_ref,
            }
        server = self.create_server(clients=self.os_admin,
                                    **create_kwargs)

        # wait for lease end
        self.wait_for_lease_end(lease_id)

        # check if the lease has been correctly terminated and
        # the instance is removed
        waiters.wait_for_server_termination(self.os_admin.servers_client,
                                            server['id'])

        # check that the host aggregate was deleted
        self.assertRaises(exceptions.NotFound,
                          self.fetch_aggregate_by_name, reservation_id)

        # check that the host is back in the freepool
        freepool = self.fetch_aggregate_by_name('freepool')
        self.assertTrue(host['host'] in freepool['hosts'])

        # check the reservation status
        lease = self.reservation_client.get_lease(lease_id)['lease']
        self.assertTrue('deleted' in
                        next(iter(lease['reservations']))['status'])

    @decorators.attr(type='smoke')
    def test_update_host_reservation(self):

        # create the host if it doesn't exist
        host = self._add_host_once()

        # create new lease and start reservation immediately
        body = self.get_lease_body('scenario-3-update', host['host'])
        lease = self.reservation_client.create_lease(body)['lease']
        lease_id = lease['id']

        # check host added to the reservation
        reservation_id = next(iter(lease['reservations']))['id']
        self.wait_until_aggregated(reservation_id, host['host'])

        # check the host aggregate for blazar
        self.fetch_aggregate_by_name(reservation_id)

        # create an instance with reservation id
        create_kwargs = {
            'scheduler_hints': {
                'reservation': reservation_id,
                },
            'image_id': CONF.compute.image_ref,
            'flavor': CONF.compute.flavor_ref,
            }
        server = self.create_server(clients=self.os_admin,
                                    wait_until=None,
                                    **create_kwargs)
        waiters.wait_for_server_status(self.os_admin.servers_client,
                                       server['id'], 'ACTIVE')

        # wait enough time for the update API to succeed
        time.sleep(75)

        # update the lease end_time
        end_time = datetime.datetime.utcnow() + datetime.timedelta(seconds=30)
        body = {
            'end_date': end_time.strftime('%Y-%m-%d %H:%M')
            }
        self.reservation_client.update_lease(lease_id,
                                             body)['lease']

        # check if the lease has been correctly terminated and
        # the instance is removed
        waiters.wait_for_server_termination(self.os_admin.servers_client,
                                            server['id'])

        # wait long enough for the lease deletion to be complete
        time.sleep(30)

        # check that the host aggregate was deleted
        self.assertRaises(exceptions.NotFound,
                          self.fetch_aggregate_by_name, reservation_id)

        # check that the host is back in the freepool
        freepool = self.fetch_aggregate_by_name('freepool')
        self.assertTrue(host['host'] in freepool['hosts'])

        # check the reservation status
        lease = self.reservation_client.get_lease(lease_id)['lease']
        self.assertTrue('deleted' in
                        next(iter(lease['reservations']))['status'])



# Copyright 2014 Intel Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg


service_available_group = cfg.OptGroup(name="service_available",
                                       title="Available OpenStack Services")

service_option = [
    cfg.BoolOpt("blazar",
                default=True,
                help="Whether or not blazar is expected to be available"),
]

resource_reservation_group = cfg.OptGroup(name='resource_reservation',
                                          title='Resource reservation service '
                                                'options')

ResourceReservationGroup = [
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the resource_reservation "
                    "service."),
    cfg.IntOpt('lease_interval',
               default=10,
               help="Time in seconds between lease status checks."),
    cfg.IntOpt('lease_end_timeout',
               default=300,
               help="Timeout in seconds to wait for a lease to finish.")
]
